@@ -17,10 +17,11 @@
 #
 # Authors:  Ralph Bean <rbean@redhat.com>
 #
-""" Fedora Messaging Client API """
+""" Federated Message Bus Client API """
 
 import inspect
 import threading
+import functools
 
 import fedmsg.core
 import fedmsg.config
@@ -31,8 +32,8 @@ __all__ = [
     'init',
     'send_message',
     'publish',
-    'subscribe',
     'destroy',
+    'tail_messages',
     '__local',
 ]
 
@@ -63,8 +64,9 @@ def init(**kw):
 
 def API_function(doc=None):
     def api_function(func):
-        scrub = inspect.getargspec(func).args
+        scrub = inspect.getfullargspec(func).args
 
+        @functools.wraps(func)
         def _wrapper(*args, **kw):
             if not hasattr(__local, '__context'):
                 config_overrides = kw.copy()
@@ -82,10 +84,7 @@ def API_function(doc=None):
             _wrapper.__doc__ = func.__doc__
         else:
             _wrapper.__doc__ = doc
-
-        _wrapper.__name__ = func.__name__
         return _wrapper
-
     return api_function
 
 
@@ -93,6 +92,7 @@ def API_function(doc=None):
 def publish(topic=None, msg=None, **kw):
     return __local.__context.publish(topic, msg, **kw)
 
+
 # This is old-school, and deprecated.
 send_message = publish
 
