@@ -26,8 +26,10 @@ import psutil
 import requests
 import threading
 import time
+import warnings
 
 import moksha.hub.api.consumer
+import six
 
 import fedmsg.crypto
 import fedmsg.encoding
@@ -35,50 +37,62 @@ from fedmsg.replay import check_for_replay
 
 
 class FedmsgConsumer(moksha.hub.api.consumer.Consumer):
-    """ Base class for fedmsg consumers.
+    """
+    Base class for fedmsg consumers.
 
-    The fedmsg consumption API is really just a thin wrapper over moksha.
-    Moksha expects consumers to:
+    This class inherits from :class:`moksha.hub.api.consumer.Consumer` and you
+    should familiarize yourself with this class as well.
 
-        * Declare themselves on the moksha.consumers python entry-point.
-        * Declare a ``consume(...)`` method.
-        * Specify a ``topic``.
+    To create a consumer, you must inherit this class and do the following:
 
-    All this class does in addition to moksha is:
+        * Declare the class on the ``moksha.consumers`` python entry-point::
 
-        * Provide a mechanism for disabling/enabling consumers by configuration
-          in a consistent way (namely, by use of ``config_key``).
+              setup(
+                  entry_points={
+                      'moksha.consumer': (
+                          'your_consumer = python.path:YourConsumerClass',
+                      ),
+                  },
+              )
 
-          If you set ``validate_signatures = False`` on your consumer, it will
-          be exempt from global validation rules.  Messages will not be
-          checked for authenticity before being handed off to your consume
-          method.  This is handy if you're developing or building a
-          special-case consumer.  The consumer used by ``fedmsg-relay``
-          (described in :doc:`commands`) sets ``validate_signatures = False``
-          so that it can transparently forward along everything and let the
-          terminal endpoints decide whether or not to consume particular
-          messages.
+        * Implement the ``consume(self, message)`` method on the class.
 
-        * Provide a mechanism for automatically validating fedmsg messages
-          with :mod:`fedmsg.crypto`.
+        * Set the attributes documented below
 
-        * Provide a mechanism to play back messages that haven't been received
-          by the hub even though emitted. To make use of this feature, you have
-          to set ``replay_name`` to some string corresponding to an endpoint in
-          the ``replay_endpoints`` dict in the configuration.
+    Attributes:
+        validate_signatures (bool): If ``False``, message authenticity will not be
+            checked. This is helpful if you're developing or building a special-case
+            consumer. For example, the consumer used by :ref:`command-relay` sets
+            ``validate_signatures = False`` so that it can transparently forward along
+            everything and let the terminal endpoints decide whether or not to consume
+            particular messages.
 
-          You must set ``config_key`` to some string.  A config value by
-          that name must be True in the config parsed by :mod:`fedmsg.config`
-          in order for the consumer to be activated.
+        topic (str or list): This attribute is required. Either a :class:`str`
+            or :class:`list` of :class:`str` that are topics that this consumer
+            is interested in receiving messages for. To receive all messages, use
+            an empty string.
+
+        config_key (str): The name of the configuration key used to enable or disable
+            this consumer. If this key is not present in the fedmsg configuration or
+            does not have a value of ``True``, :ref:`command-hub` will not run the
+            consumer.
+
+        replay_name (str): The name of the replay endpoint where the system should
+            query for playback in case of missed messages. It must match a service
+            key in :ref:`conf-replay-endpoints`. This attribute is optional.
+
+    Args:
+        hub (moksha.hub.hub.MokshaCentralHub): The Moksha Hub that is initializing this
+            consumer.
     """
 
-    validate_signatures = False
+    validate_signatures = None
     config_key = None
 
     def __init__(self, hub):
         module = inspect.getmodule(self).__name__
         name = self.__class__.__name__
-        self.log = logging.getLogger("fedmsg")
+        self.log = logging.getLogger(__name__)
 
         if not self.config_key:
             raise ValueError("%s:%s must declare a 'config_key'" % (
@@ -98,9 +112,9 @@ class FedmsgConsumer(moksha.hub.api.consumer.Consumer):
         super(FedmsgConsumer, self).__init__(hub)
 
         # Now, re-get our logger to override the one moksha assigns us.
-        self.log = logging.getLogger("fedmsg")
+        self.log = logging.getLogger(__name__)
 
-        if self.validate_signatures:
+        if self.validate_signatures is None:
             self.validate_signatures = self.hub.config['validate_signatures']
 
         if hasattr(self, "replay_name"):
@@ -161,7 +175,7 @@ class FedmsgConsumer(moksha.hub.api.consumer.Consumer):
             return
 
         last = data['message']['body']
-        if isinstance(last, basestring):
+        if isinstance(last, str):
             last = json.loads(last)
 
         then = last['timestamp']
@@ -169,6 +183,11 @@ class FedmsgConsumer(moksha.hub.api.consumer.Consumer):
 
         retrieved = 0
         for message in self.get_datagrepper_results(then, now):
+            # Take the messages from datagrepper and remove any keys that were
+            # artificially added to the message.  The presence of these would
+            # otherwise cause message crypto validation to fail.
+            message = fedmsg.crypto.utils.fix_datagrepper_message(message)
+
             if message['msg_id'] != last['msg_id']:
                 retrieved = retrieved + 1
                 self.incoming.put(dict(body=message, topic=message['topic']))
@@ -203,31 +222,93 @@ class FedmsgConsumer(moksha.hub.api.consumer.Consumer):
                         break
 
     def validate(self, message):
-        """ This needs to raise an exception, caught by moksha. """
+        """
+        Validate the message before the consumer processes it.
 
+        This needs to raise an exception, caught by moksha.
+
+        Args:
+            message (dict): The message as a dictionary. This must, at a minimum,
+                contain the 'topic' key with a unicode string value and 'body' key
+                with a dictionary value. However, the message might also be an object
+                with a ``__json__`` method that returns a dict with a 'body' key that
+                can be a unicode string that is JSON-encoded.
+
+        Raises:
+            RuntimeWarning: If the message is not valid.
+            UnicodeDecodeError: If the message body is not unicode or UTF-8 and also
+                happens to contain invalid UTF-8 binary.
+        """
         if hasattr(message, '__json__'):
             message = message.__json__()
-            if isinstance(message['body'], basestring):
+            if isinstance(message['body'], six.text_type):
                 message['body'] = json.loads(message['body'])
+            elif isinstance(message['body'], six.binary_type):
+                # Try to decode the message body as UTF-8 since it's very likely
+                # that that was the encoding used. This API should eventually only
+                # accept unicode strings inside messages. If a UnicodeDecodeError
+                # happens, let that bubble up.
+                warnings.warn('Message body is not unicode', DeprecationWarning)
+                message['body'] = json.loads(message['body'].decode('utf-8'))
 
-        # We assume these match inside fedmsg.crypto, so we should enforce it.
-        if not message['topic'] == message['body']['topic']:
-            raise RuntimeWarning("Topic envelope mismatch.")
+        # Massage STOMP messages into a more compatible format.
+        if not isinstance(message['body'], dict) or 'topic' not in message['body']:
+            message['body'] = {
+                'topic': message.get('topic'),
+                'msg': message['body'],
+            }
 
         # If we're not validating, then everything is valid.
         # If this is turned on globally, our child class can override it.
         if not self.validate_signatures:
             return
 
+        # We assume these match inside fedmsg.crypto, so we should enforce it.
+        if not message['topic'] == message['body']['topic']:
+            raise RuntimeWarning("Topic envelope mismatch.")
+
         if not fedmsg.crypto.validate(message['body'], **self.hub.config):
             raise RuntimeWarning("Failed to authn message.")
 
     def _consume(self, message):
+        """ Called when a message is consumed.
+
+        This private method handles some administrative setup and teardown
+        before calling the public interface `consume` typically implemented
+        by a subclass.
+
+        When `moksha.blocking_mode` is set to `False` in the config, this
+        method always returns `None`.  The argued message is stored in an
+        internal queue where the consumer's worker threads should eventually
+        pick it up.
+
+        When `moksha.blocking_mode` is set to `True` in the config, this
+        method should return True or False, indicating whether the message
+        was handled or not.  Specifically, in the event that the inner
+        `consume` method raises an exception of any kind, this method
+        should return `False` indicating that the message was not
+        successfully handled.
+
+        Args:
+            message (dict): The message as a dictionary.
+
+        Returns:
+            bool: Should be interpreted as whether or not the message was
+            handled by the consumer, or `None` if `moksha.blocking_mode` is
+            set to False.
+        """
+
         try:
             self.validate(message)
         except RuntimeWarning as e:
             self.log.warn("Received invalid message {0}".format(e))
             return
+
+        # Pass along headers if present.  May be useful to filters or
+        # fedmsg.meta routines.
+        if isinstance(message, dict) and 'headers' in message and 'body' in message:
+            message['body']['headers'] = message['headers']
+
         if hasattr(self, "replay_name"):
             for m in check_for_replay(
                     self.replay_name, self.name_to_seq_id,
@@ -235,11 +316,11 @@ class FedmsgConsumer(moksha.hub.api.consumer.Consumer):
 
                 try:
                     self.validate(m)
-                    super(FedmsgConsumer, self)._consume(m)
+                    return super(FedmsgConsumer, self)._consume(m)
                 except RuntimeWarning as e:
                     self.log.warn("Received invalid message {}".format(e))
         else:
-            super(FedmsgConsumer, self)._consume(message)
+            return super(FedmsgConsumer, self)._consume(message)
 
     def pre_consume(self, message):
         self.save_status(dict(
