@@ -28,6 +28,7 @@ import fedmsg.meta
 from fedmsg.meta import _
 
 import copy
+import functools
 import re
 import time
 import pygments
@@ -65,6 +66,16 @@ mirc_colors = {
 }
 
 
+def _default_link_shortener(url):
+    dagd = 'https://da.gd/s'
+    try:
+        resp = requests.get(dagd, params=dict(url=url), timeout=3)
+        return resp.text.strip()
+    except:  # noqa: E722
+        log.exception("Failed to shorten %r" % url)
+        return url
+
+
 def ircprettify(title, subtitle, link="", config=None):
     def markup(s, color):
         return "\x03%i%s\x03" % (mirc_colors[color], s)
@@ -77,7 +88,8 @@ def ircprettify(title, subtitle, link="", config=None):
         link = ""
 
     color_lookup = config.get('irc_color_lookup', {})
-    title_color = color_lookup.get(title.split('.')[0], "light grey")
+    default_color = config.get('irc_default_color', 'light grey')
+    title_color = color_lookup.get(title.split('.')[0], default_color)
     title = markup(title, title_color)
 
     fmt = u"{title} -- {subtitle} {link}"
@@ -147,7 +159,7 @@ class Fedmsg2IRCFactory(protocol.ClientFactory):
     protocol = make_irc_client
 
     def __init__(self, channel, nickname, filters,
-                 pretty, terse, short, rate, parent_consumer):
+                 pretty, terse, short, rate, parent_consumer, ready):
         self.channel = channel
         self.nickname = nickname
         self.filters = filters
@@ -156,23 +168,34 @@ class Fedmsg2IRCFactory(protocol.ClientFactory):
         self.short = short
         self.rate = rate
         self.parent_consumer = parent_consumer
-        self.log = logging.getLogger("moksha.hub")
+        self.ready = ready
+        self.log = logging.getLogger(__name__)
+
+    def startedConnecting(self, connector):
+        if self.ready:
+            # If we're joining 12 channels, join one of them first.  Once
+            # joining, wait five seconds and start joining the second one.
+            # That one should trigger joining the third one...
+            log.info("%s scheduling conn for next client" % self.nickname)
+            reactor.callLater(5, self.ready)
+            # Un set this so we don't trigger it again later on a reconnect...
+            self.ready = None
 
     def clientConnectionLost(self, connector, reason):
         if self.parent_consumer.die:
-            self.log.info("Lost connection.  Not reconnecting to IRC.")
+            log.info("Lost connection.  Not reconnecting to IRC.")
             return
 
-        self.log.warning("Lost connection (%s), reconnecting." % (reason,))
+        log.warning("Lost connection (%s), reconnecting." % (reason,))
         self.parent_consumer.del_irc_clients(factory=self)
         connector.connect()
 
     def clientConnectionFailed(self, connector, reason):
         if self.parent_consumer.die:
-            self.log.info("Failed connection.  Not reconnecting to IRC.")
+            log.info("Failed connection.  Not reconnecting to IRC.")
             return
 
-        self.log.error("Could not connect: %s, retry in 60s" % (reason,))
+        log.error("Could not connect: %s, retry in 60s" % (reason,))
         self.parent_consumer.del_irc_clients(factory=self)
         reactor.callLater(60, connector.connect)
 
@@ -195,12 +218,14 @@ class IRCBotConsumer(FedmsgConsumer):
             return
 
         irc_settings = hub.config.get('irc')
+        callback = None  # Keep track of the last factory we created
         for settings in irc_settings:
             network = settings.get('network', 'irc.freenode.net')
             port = settings.get('port', 6667)
+            use_ssl = settings.get('ssl', False)
             channel = settings.get('channel', None)
             if not channel:
-                self.log.error("No channel specified.  Ignoring entry.")
+                log.error("No channel specified.  Ignoring entry.")
                 continue
 
             if not channel.startswith("#"):
@@ -216,8 +241,30 @@ class IRCBotConsumer(FedmsgConsumer):
             filters = self.compile_filters(settings.get('filters', None))
 
             factory = Fedmsg2IRCFactory(
-                channel, nickname, filters, pretty, terse, short, rate, self)
-            reactor.connectTCP(network, port, factory, timeout=timeout)
+                channel, nickname, filters,
+                pretty, terse, short, rate,
+                self, ready=callback,
+            )
+            if use_ssl:
+                from twisted.internet import ssl
+                callback = functools.partial(
+                    reactor.connectSSL,
+                    network, port, factory,
+                    ssl.ClientContextFactory(),
+                    timeout=timeout,
+                )
+            else:
+                callback = functools.partial(
+                    reactor.connectTCP,
+                    network, port, factory,
+                    timeout=timeout,
+                )
+
+        # Call only the very last one.
+        # When it is done, it will call the second to last one, which when it
+        # is done will call the third to last one, etc..
+        log.info("Scheduling connection: %r" % callback)
+        callback()
 
     def add_irc_client(self, client):
         self.irc_clients.append(client)
@@ -271,9 +318,10 @@ class IRCBotConsumer(FedmsgConsumer):
                     link = fedmsg.meta.msg2link(msg, **self.hub.config)
 
                 if link and short:
-                    dagd = 'http://da.gd/s'
-                    resp = requests.get(dagd, params=dict(url=link))
-                    link = resp.text.strip()
+                    if callable(short):
+                        link = short(link)
+                    else:
+                        link = _default_link_shortener(link)
 
                 return ircprettify(
                     title=title,
@@ -324,7 +372,7 @@ class IRCBotConsumer(FedmsgConsumer):
                 backlog = self.incoming.qsize()
                 if backlog and (backlog % 20) == 0:
                     warning = "* backlogged by %i messages" % backlog
-                    self.log.warning(warning)
+                    log.warning(warning)
                     send(client.factory.channel, warning.encode('utf-8'))
 
     def stop(self):
