@@ -17,10 +17,10 @@
 #
 # Authors:  Ralph Bean <rbean@redhat.com>
 #
-import fedmsg
 import logging
 
 from fedmsg.consumers import FedmsgConsumer
+from fedmsg import crypto
 
 log = logging.getLogger(__name__)
 
@@ -38,10 +38,10 @@ class RelayConsumer(FedmsgConsumer):
         self.validate_signatures = False
 
     def consume(self, msg):
-        ## FIXME - for some reason twisted is screwing up fedmsg.
-        #fedmsg.__context.publisher.send_multipart(
+        # FIXME - for some reason twisted is screwing up fedmsg.
+        # fedmsg.__context.publisher.send_multipart(
         #    [msg['topic'], fedmsg.encoding.dumps(msg['body'])]
-        #)
+        # )
         #
         # We have to do this instead.  This works for the fedmsg-relay service
         # since it doesn't need to do any formatting of the message.  It just
@@ -53,3 +53,37 @@ class RelayConsumer(FedmsgConsumer):
 
         log.debug("Got message %r" % msg)
         self.hub.send_message(topic=msg['topic'], message=msg['body'])
+
+
+class SigningRelayConsumer(RelayConsumer):
+    """
+    A relay that signs messages it relays with x509 certificates.
+
+    The key pair used for message signing is configured by setting the ``signing_relay``
+    key in the ``certnames`` configuration dictionary to the key pair name inside of
+    the configured ``ssldir``. See the configuration documentation for more information.
+    """
+
+    def __init__(self, *args, **kwargs):
+        """
+        Initialize the relay.
+
+        All arguments are passed directly on to the :class:`RelayConsumer`.
+        """
+        super(SigningRelayConsumer, self).__init__(*args, **kwargs)
+
+        try:
+            self.hub.config['certname'] = self.hub.config['certnames']['signing_relay']
+        except KeyError:
+            log.error('The signing relay requires that the certificate name is in '
+                      'the "certnames" dictionary using the "signing_relay" key')
+
+    def consume(self, msg):
+        """
+        Sign the message prior to sending the message.
+
+        Args:
+            msg (dict): The message to sign and relay.
+        """
+        msg['body'] = crypto.sign(msg['body'], **self.hub.config)
+        super(SigningRelayConsumer, self).consume(msg)
