@@ -47,13 +47,19 @@ from fedmsg.replay import check_for_replay
 import logging
 
 
+class ValidationError(Exception):
+    """ Error used internally to represent a validation failure. """
+    def __init__(self, msg):
+        self.msg = msg
+
+
 class FedMsgContext(object):
     # A counter for messages sent.
     _i = 0
 
     def __init__(self, **config):
         super(FedMsgContext, self).__init__()
-        self.log = logging.getLogger("fedmsg")
+        self.log = logging.getLogger(__name__)
 
         self.c = config
         self.hostname = socket.gethostname().split('.', 1)[0]
@@ -69,12 +75,12 @@ class FedMsgContext(object):
             module_name = guess_calling_module(default="fedmsg")
             config["name"] = module_name + '.' + self.hostname
 
-            if any(map(config["name"].startswith, ['fedmsg'])):
+            if config["name"].startswith('fedmsg'):
                 config["name"] = None
 
         # Do a little special-case mangling.  We never want to "listen" to the
         # relay_inbound address, but in the special case that we want to emit
-        # our messages there, we add it to the :term:`endpoints` dict so that
+        # our messages there, we add it to the :ref:`conf-endpoints` dict so that
         # the code below where we "Actually set up our publisher" can be
         # simplified.  See Issue #37 - https://bit.ly/KN6dEK
         if config.get('active', False):
@@ -83,10 +89,11 @@ class FedMsgContext(object):
                 config['endpoints'][name] = config[name]
             except KeyError:
                 raise KeyError("Could not find endpoint for fedmsg-relay."
-                              " Try installing fedmsg-relay.")
+                               " Try installing fedmsg-relay.")
 
-        # Actually set up our publisher
+        # Actually set up our publisher, but only if we're configured for zmq.
         if (
+            config.get('zmq_enabled', True) and
             not config.get("mute", False) and
             config.get("name", None) and
             config.get("endpoints", None) and
@@ -114,7 +121,7 @@ class FedMsgContext(object):
             # endpoint.
             _established = False
             for endpoint in config['endpoints'][config['name']]:
-
+                self.log.debug("Trying to %s to %s" % (method, endpoint))
                 if method == 'bind':
                     endpoint = "tcp://*:{port}".format(
                         port=endpoint.rsplit(':')[-1]
@@ -145,13 +152,18 @@ class FedMsgContext(object):
         elif config.get('mute', False):
             # Our caller doesn't intend to send any messages.  Pass silently.
             pass
-        else:
+        elif config.get('zmq_enabled', True):
             # Something is wrong.
             warnings.warn(
-                "fedmsg is not configured to send any messages "
+                "fedmsg is not configured to send any zmq messages "
                 "for name %r" % config.get("name", None))
+        else:
+            # We're not configured to send zmq messages, but zmq_enabled is
+            # False, so no need to warn the user.
+            pass
 
         # Cleanup.  See https://bit.ly/SaGeOr for discussion.
+        # arg signature - weakref.ref(object [, callback])
         weakref.ref(threading.current_thread(), self.destroy)
 
         # Sleep just to make sure that the socket gets set up before anyone
@@ -162,6 +174,8 @@ class FedMsgContext(object):
         """ Destroy a fedmsg context """
 
         if getattr(self, 'publisher', None):
+            self.log.debug("closing fedmsg publisher")
+            self.log.debug("sent %i messages" % self._i)
             self.publisher.close()
             self.publisher = None
 
@@ -170,20 +184,26 @@ class FedMsgContext(object):
             self.context = None
 
     def send_message(self, topic=None, msg=None, modname=None):
-        warnings.warn(".send_message is deprecated.", DeprecationWarning)
+        warnings.warn(
+            ".send_message is deprecated. Use .publish", DeprecationWarning)
 
         return self.publish(topic, msg, modname)
 
-    def publish(self, topic=None, msg=None, modname=None, **kw):
-        """ Send a message over the publishing zeromq socket.
+    def publish(self, topic=None, msg=None, modname=None,
+                pre_fire_hook=None, **kw):
+        """
+        Send a message over the publishing zeromq socket.
 
-          >>> import fedmsg
-          >>> fedmsg.publish(topic='testing', modname='test', msg={
-          ...     'test': "Hello World",
-          ... })
+            >>> import fedmsg
+            >>> fedmsg.publish(topic='testing', modname='test', msg={
+            ...     'test': "Hello World",
+            ... })
 
         The above snippet will send the message ``'{test: "Hello World"}'``
-        over the ``<topic_prefix>.dev.test.testing`` topic.
+        over the ``<topic_prefix>.dev.test.testing`` topic. The fully qualified
+        topic of a message is constructed out of the following pieces:
+
+            <:ref:`conf-topic-prefix`>.<:ref:`conf-environment`>.<``modname``>.<``topic``>
 
         This function (and other API functions) do a little bit more
         heavy lifting than they let on.  If the "zeromq context" is not yet
@@ -191,18 +211,6 @@ class FedMsgContext(object):
         store it as :data:`fedmsg.__local.__context` before anything else is
         done.
 
-        The ``modname`` argument will be omitted in most use cases.  By
-        default, ``fedmsg`` will try to guess the name of the module that
-        called it and use that to produce an intelligent topic.  Specifying
-        ``modname`` explicitly overrides this behavior.
-
-        The fully qualified topic of a message is constructed out of the
-        following pieces:
-
-         <:term:`topic_prefix`>.<:term:`environment`>.<``modname``>.<``topic``>
-
-        ----
-
         **An example from Fedora Tagger -- SQLAlchemy encoding**
 
         Here's an example from
@@ -229,8 +237,6 @@ class FedMsgContext(object):
         stripped it down to just ``fedoratagger`` for the final topic of
         ``org.fedoraproject.{dev,stg,prod}.fedoratagger.tag.update``.
 
-        ----
-
         **Shell Usage**
 
         You could also use the ``fedmsg-logger`` from a shell script like so::
@@ -238,6 +244,24 @@ class FedMsgContext(object):
             $ echo "Hello, world." | fedmsg-logger --topic testing
             $ echo '{"foo": "bar"}' | fedmsg-logger --json-input
 
+        :param topic: The message topic suffix. This suffix is joined to the
+            configured topic prefix (e.g. ``org.fedoraproject``), environment
+            (e.g. ``prod``, ``dev``, etc.), and modname.
+        :type topic: unicode
+        :param msg: A message to publish. This message will be JSON-encoded
+            prior to being sent, so the object must be composed of JSON-
+            serializable data types. Please note that if this is already a
+            string JSON serialization will be applied to that string.
+        :type msg: dict
+        :param modname: The module name that is publishing the message. If this
+            is omitted, ``fedmsg`` will try to guess the name of the module
+            that called it and use that to produce an intelligent topic.
+            Specifying ``modname`` explicitly overrides this behavior.
+        :type modname: unicode
+        :param pre_fire_hook: A callable that will be called with a single
+            argument -- the dict of the constructed message -- just before it
+            is handed off to ZeroMQ for publication.
+        :type pre_fire_hook: function
         """
 
         topic = topic or 'unspecified'
@@ -282,7 +306,8 @@ class FedMsgContext(object):
 
                 self.c['certname'] = self.c['certnames'][cert_index]
             else:
-                self.c['gpg_signing_key'] = self.c['gpg_keys'][cert_index]
+                if 'gpg_signing_key' not in self.c:
+                    self.c['gpg_signing_key'] = self.c['gpg_keys'][self.hostname]
 
         if self.c.get('sign_messages', False):
             msg = fedmsg.crypto.sign(msg, **self.c)
@@ -292,30 +317,70 @@ class FedMsgContext(object):
             # Add the seq_id field
             msg = store.add(msg)
 
-        self.publisher.send_multipart(
-            [topic, fedmsg.encoding.dumps(msg).encode('utf-8')],
-            flags=zmq.NOBLOCK,
-        )
+        if pre_fire_hook:
+            pre_fire_hook(msg)
+
+        # We handle zeromq publishing ourselves.  But, if that is disabled,
+        # defer to the moksha' hub's twisted reactor to send messages (if
+        # available).
+        if self.c.get('zmq_enabled', True):
+            self.publisher.send_multipart(
+                [topic, fedmsg.encoding.dumps(msg).encode('utf-8')],
+                flags=zmq.NOBLOCK,
+            )
+        else:
+            # Perhaps we're using STOMP or AMQP?  Let moksha handle it.
+            import moksha.hub
+            # First, a quick sanity check.
+            if not getattr(moksha.hub, '_hub', None):
+                raise AttributeError("Unable to publish non-zeromq msg "
+                                     "without moksha-hub initialization.")
+            # Let moksha.hub do our work.
+            moksha.hub._hub.send_message(
+                topic=topic,
+                message=fedmsg.encoding.dumps(msg).encode('utf-8'),
+                jsonify=False,
+            )
 
     def tail_messages(self, topic="", passive=False, **kw):
-        """ Tail messages on the bus.
+        """
+        Subscribe to messages published on the sockets listed in :ref:`conf-endpoints`.
 
-        Generator that yields tuples of the form:
-        ``(name, endpoint, topic, message)``
+        Args:
+            topic (six.text_type): The topic to subscribe to. The default is to
+                subscribe to all topics.
+            passive (bool): If ``True``, bind to the :ref:`conf-endpoints` sockets
+                instead of connecting to them. Defaults to ``False``.
+            **kw: Additional keyword arguments. Currently none are used.
+
+        Yields:
+            tuple: A 4-tuple in the form (name, endpoint, topic, message).
         """
 
+        if not self.c.get('zmq_enabled', True):
+            raise ValueError("fedmsg.tail_messages() is only available for "
+                             "zeromq.  Use the hub-consumer approach for "
+                             "STOMP or AMQP support.")
+
+        poller, subs = self._create_poller(topic=topic, passive=False, **kw)
+        try:
+            for msg in self._poll(poller, subs):
+                yield msg
+        finally:
+            self._close_subs(subs)
+
+    def _create_poller(self, topic="", passive=False, **kw):
         # TODO -- do the zmq_strict logic dance with "topic" here.
         # It is buried in moksha.hub, but we need it to work the same way
         # here.
 
         # TODO -- the 'passive' here and the 'active' are ambiguous.  They
         # don't actually mean the same thing.  This should be resolved.
-        method = passive and 'bind' or 'connect'
+        method = (passive and 'bind') or 'connect'
 
         failed_hostnames = []
         subs = {}
-        watched_names = {}
-        for _name, endpoint_list in self.c['endpoints'].iteritems():
+        for _name, endpoint_list in six.iteritems(self.c['endpoints']):
 
             # You never want to actually subscribe to this thing, but sometimes
             # it appears in the endpoints list due to a hack where it gets
@@ -335,14 +400,14 @@ class FedMsgContext(object):
                 if hostname != '*':
                     try:
                         socket.gethostbyname_ex(hostname)
-                    except:
+                    except:  # noqa: E722
                         failed_hostnames.append(hostname)
                         self.log.warn("Couldn't resolve %r" % hostname)
                         continue
 
                 # OK, sanity checks pass.  Create the subscriber and connect.
                 subscriber = self.context.socket(zmq.SUB)
-                subscriber.setsockopt(zmq.SUBSCRIBE, topic)
+                subscriber.setsockopt(zmq.SUBSCRIBE, topic.encode('utf-8'))
 
                 set_high_water_mark(subscriber, self.c)
                 set_tcp_keepalive(subscriber, self.c)
@@ -350,50 +415,65 @@ class FedMsgContext(object):
 
                 getattr(subscriber, method)(endpoint)
                 subs[subscriber] = (_name, endpoint)
-            if _name in self.c.get("replay_endpoints", {}):
-                # At first we don't know where the sequence is at.
-                watched_names[_name] = -1
 
         # Register the sockets we just built with a zmq Poller.
         poller = zmq.Poller()
         for subscriber in subs:
             poller.register(subscriber, zmq.POLLIN)
 
-        # TODO -- what if user wants to pass in validate_signatures in **kw?
-        validate = self.c.get('validate_signatures', False)
+        return (poller, subs)
+
+    def _poll(self, poller, subs):
+        watched_names = {}
+        for name, _ in subs.values():
+            if name in self.c.get("replay_endpoints", {}):
+                # At first we don't know where the sequence is at.
+                watched_names[name] = -1
 
         # Poll that poller.  This is much more efficient than it used to be.
-        try:
-            while True:
-                sockets = dict(poller.poll())
-                for s in sockets:
-                    _name, ep = subs[s]
-                    _topic, message = s.recv_multipart()
-                    msg = fedmsg.encoding.loads(message)
-                    if not validate or fedmsg.crypto.validate(msg, **self.c):
-                        # If there is even a slight change of replay, use
-                        # check_for_replay
-                        if len(self.c.get('replay_endpoints', {})) > 0:
-                            for m in check_for_replay(
-                                    _name, watched_names,
-                                    msg, self.c, self.context):
+        while True:
+            sockets = dict(poller.poll())
+            for s in sockets:
+                name, ep = subs[s]
+                try:
+                    yield self._run_socket(s, name, ep, watched_names)
+                except ValidationError as e:
+                    warnings.warn("!! invalid message received: %r" % e.msg)
 
-                                # Revalidate all the replayed messages.
-                                if not validate or \
-                                        fedmsg.crypto.validate(m, **self.c):
-                                    yield _name, ep, m['topic'], m
-                                else:
-                                    warnings.warn("!! invalid message " +
-                                                  "received: %r" % msg)
-                        else:
-                            yield _name, ep, _topic, msg
+    def _run_socket(self, sock, name, ep, watched_names=None):
+        if watched_names is None:
+            watched_names = {}
+
+        validate = self.c.get('validate_signatures', False)
+
+        # Grab the data off the zeromq internal queue
+        _topic, message = sock.recv_multipart()
+
+        # zmq hands us byte strings, so let's convert to unicode asap
+        _topic, message = _topic.decode('utf-8'), message.decode('utf-8')
+
+        # Now, decode the JSON body into a dict.
+        msg = fedmsg.encoding.loads(message)
+
+        if not validate or fedmsg.crypto.validate(msg, **self.c):
+            # If there is even a slight change of replay, use
+            # check_for_replay
+            if len(self.c.get('replay_endpoints', {})) > 0:
+                for m in check_for_replay(
+                        name, watched_names,
+                        msg, self.c, self.context):
+
+                    # Revalidate all the replayed messages.
+                    if not validate or \
+                            fedmsg.crypto.validate(m, **self.c):
+                        return name, ep, m['topic'], m
                     else:
-                        # Else.. we are supposed to be validating, but the
-                        # message failed validation.
+                        raise ValidationError(msg)
+            else:
+                return (name, ep, _topic, msg)
+        else:
+            raise ValidationError(msg)
 
-                        # Warn, but don't throw an exception.  Keep tailing.
-                        warnings.warn("!! invalid message received: %r" % msg)
-
-        finally:
-            for subscriber in subs:
-                subscriber.close()
+    def _close_subs(self, subs):
+        for subscriber in subs:
+            subscriber.close()
