@@ -2,6 +2,11 @@ import os
 import socket
 import fedmsg.config
 
+try:
+    import unittest2 as unittest
+except ImportError:
+    import unittest
+
 
 def load_config(name='fedmsg-test-config.py'):
     here = os.path.sep.join(__file__.split(os.path.sep)[:-1])
@@ -29,3 +34,7 @@ def load_config(name='fedmsg-test-config.py'):
         config['endpoints'][hub_name]
     )
     return config
+
+
+requires_network = unittest.skipIf(
+    not os.environ.get('FEDMSG_NETWORK'), "Skip test since we don't have network")
