@@ -26,10 +26,10 @@ SEP = os.path.sep
 here = os.getcwd()
 hostname = socket.gethostname().split('.', 1)[0]
 
-ssl_enabled_for_tests = True
 try:
-    import M2Crypto
-    import m2ext
+    import M2Crypto             # noqa: F401
+    import m2ext                # noqa: F401
+    ssl_enabled_for_tests = True
 except ImportError:
     ssl_enabled_for_tests = False
 
@@ -40,7 +40,8 @@ gpg_key_unittest = 'FBDA 92E4 338D FFD9 EB83  F8F6 3FBD B725 DA19 B4EC'
 gpg_key_main = 'FBDA 92E4 338D FFD9 EB83  F8F6 3FBD B725 DA19 B4EC'
 
 config = dict(
-    topic_prefix="com.test_prefix",
+    topic_prefix="org.fedoraproject",
+    topic_prefix_re="^org\\.fedoraproject\\.(dev|stg|prod)",
     endpoints={
         "unittest.%s" % hostname: [
             "tcp://*:%i" % (port + 1),
@@ -64,7 +65,7 @@ config = dict(
         # Guarantee that we don't fall over with a bogus endpoint.
         "blah.%s": "tcp://www.flugle.horn:88",
     },
-    relay_inbound=["tcp://127.0.0.1:%i" % (port - 1)],
+    relay_inbound="tcp://127.0.0.1:%i" % (port - 1),
     replay_endpoints={
         'unittest.%s' % hostname: "tcp://127.0.0.1:%i" % (port + 1),
         'unittest2.%s' % hostname: "tcp://127.0.0.1:%i" % (port + 2),
@@ -87,7 +88,7 @@ config = dict(
 
     crl_location="http://threebean.org/fedmsg-tests/crl.pem",
     crl_cache="/tmp/crl.pem",
-    crl_cache_expiry=10,
+    crl_cache_expiry=3600,
 
     certnames={
         "unittest.%s" % hostname: "shell-app01.phx2.fedoraproject.org",
@@ -95,7 +96,7 @@ config = dict(
         "__main__.%s" % hostname: "shell-app01.phx2.fedoraproject.org",
         # In prod/stg, map hostname to the name of the cert in ssldir.
         # Unfortunately, we can't use socket.getfqdn()
-        #"app01.stg": "app01.stg.phx2.fedoraproject.org",
+        # "app01.stg": "app01.stg.phx2.fedoraproject.org",
     },
     gpg_keys={
         "unittest.%s" % hostname: gpg_key_unittest,
