@@ -20,53 +20,55 @@
 """ Tests for fedmsg.meta """
 
 import inspect
+import functools
 import os
-import unittest
 import textwrap
-
-from nose import SkipTest
-from nose.tools import eq_
-
 try:
-    from nose.tools.nontrivial import make_decorator
+    import unittest2 as unittest
 except ImportError:
-    # It lives here in older versions of nose (el6)
-    from nose.tools import make_decorator
+    import unittest
 
 import fedmsg.meta
 
 
+class UnspecifiedType(object):
+    """ A sentinel used to identify which expectations are not specified. """
+    def __str__(self):
+        return "Unspecified"
+
+    def __unicode__(self):
+        return u"Unspecified"
+
+    def __repr__(self):
+        return "Unspecified"
+
+
+Unspecified = UnspecifiedType()
+
+
 def skip_on(attributes):
     """ A test decorator that will skip if any of the named attributes
-    are left unspecified (are None-valued).
+    are left unspecified.
     """
     def wrapper(func):
-        @make_decorator(func)
+        @functools.wraps(func)
         def inner(self):
             for attr in attributes:
-                if getattr(self, attr) is None:
-                    raise SkipTest("%r left unspecified" % attr)
+                if getattr(self, attr) is Unspecified:
+                    raise unittest.SkipTest("%r left unspecified" % attr)
             return func(self)
         return inner
     return wrapper
 
 
-def skip_if_fedmsg_meta_FI_is_present(f):
-    """ A test decorator that will skip if fedmsg_meta_fedora_infrastructure
-    is installed.
+try:
+    import fedmsg_meta_fedora_infrastructure  # noqa: F401
+    _fedmsg_meta_fi = True
+except ImportError:
+    _fedmsg_meta_fi = False
 
-    The presence of that module will screw up some tests.
-    """
-    def _wrapper(self, *args, **kw):
-        try:
-            import fedmsg_meta_fedora_infrastructure
-            raise SkipTest("fedmsg_meta_FI is present")
-        except ImportError:
-            pass
 
-        return f(self, *args, **kw)
-
-    return make_decorator(f)(_wrapper)
+skip_if_fedmsg_meta_FI_is_present = unittest.skipIf(_fedmsg_meta_fi, "fedmsg_meta_FI is present")
 
 
 class TestForWarning(unittest.TestCase):
@@ -77,7 +79,7 @@ class TestForWarning(unittest.TestCase):
             invalidate_cache=True,
         )
         self.config['topic_prefix'] = 'org.fedoraproject'
-        self.config['topic_prefix_re'] = '^org\.fedoraproject\.(dev|stg|prod)'
+        self.config['topic_prefix_re'] = r'^org\.fedoraproject\.(dev|stg|prod)'
 
     @skip_if_fedmsg_meta_FI_is_present
     def test_for_no_plugins(self):
@@ -90,9 +92,10 @@ class TestForWarning(unittest.TestCase):
         expected = 'No fedmsg.meta plugins found.  fedmsg.meta.msg2* crippled'
         original = fedmsg.meta.log.warn
         try:
+            fedmsg.meta.processors = []
             fedmsg.meta.log.warn = mocked_warning
             fedmsg.meta.make_processors(**self.config)
-            eq_(messages, [expected])
+            self.assertEqual(messages, [expected])
         finally:
             fedmsg.meta.log.warn = original
 
@@ -105,7 +108,7 @@ class TestProcessorRegex(unittest.TestCase):
             invalidate_cache=True,
         )
         self.config['topic_prefix'] = 'org.fedoraproject'
-        self.config['topic_prefix_re'] = '^org\.fedoraproject\.(dev|stg|prod)'
+        self.config['topic_prefix_re'] = r'^org\.fedoraproject\.(dev|stg|prod)'
 
         class MyGitProcessor(fedmsg.meta.base.BaseProcessor):
             __name__ = 'git'
@@ -138,23 +141,24 @@ class TestProcessorRegex(unittest.TestCase):
             'topic': 'org.fedoraproject.dev.git',
         }
         result = self.proc.handle_msg(fake_message, **self.config)
-        assert result is "", "Proc said it couldn't handle the msg."
+        assert result == "", "Proc said it couldn't handle the msg."
 
 
 class Base(unittest.TestCase):
-    msg = None
-    expected_title = None
-    expected_subti = None
-    expected_markup = None
-    expected_link = None
-    expected_icon = None
-    expected_secondary_icon = None
-    expected_usernames = None
-    expected_packages = None
-    expected_objects = None
-    expected_emails = None
-    expected_avatars = None
-    expected_long_form = None
+    msg = Unspecified
+    expected_title = Unspecified
+    expected_subti = Unspecified
+    expected_subjective = Unspecified
+    expected_link = Unspecified
+    expected_icon = Unspecified
+    expected_secondary_icon = Unspecified
+    expected_usernames = Unspecified
+    expected_agent = Unspecified
+    expected_packages = Unspecified
+    expected_objects = Unspecified
+    expected_emails = Unspecified
+    expected_avatars = Unspecified
+    expected_long_form = Unspecified
 
     def setUp(self):
         dirname = os.path.abspath(os.path.dirname(__file__))
@@ -163,7 +167,7 @@ class Base(unittest.TestCase):
             invalidate_cache=True,
         )
         self.config['topic_prefix'] = 'org.fedoraproject'
-        self.config['topic_prefix_re'] = '^org\.fedoraproject\.(dev|stg|prod)'
+        self.config['topic_prefix_re'] = r'^org\.fedoraproject\.(dev|stg|prod)'
         fedmsg.meta.make_processors(**self.config)
 
         self.maxDiff = None
@@ -177,10 +181,10 @@ class Base(unittest.TestCase):
                 self.assertMultiLineEqual(actual, expected)
             else:
                 self.assertEquals(actual, expected)
-        except:
-            print "Failed at:"
-            print " ", self
-            print " ", os.path.relpath(inspect.getfile(type(self))[:-1])
+        except:  # noqa: E722
+            print("Failed at:")
+            print(" ", self)
+            print(" ", os.path.relpath(inspect.getfile(type(self))[:-1]))
             raise
 
     @skip_on(['msg', 'expected_title'])
@@ -189,13 +193,6 @@ class Base(unittest.TestCase):
         actual_title = fedmsg.meta.msg2title(self.msg, **self.config)
         self._equals(actual_title, self.expected_title)
 
-    @skip_on(['msg', 'expected_markup'])
-    def test_markup(self):
-        """ Does fedmsg.meta produce the right html when markup=True? """
-        actual_markup = fedmsg.meta.msg2subtitle(
-            self.msg, markup=True, **self.config)
-        self._equals(actual_markup, self.expected_markup)
-
     @skip_on(['msg', 'expected_long_form'])
     def test_long_form(self):
         """ Does fedmsg.meta produce the expected long form text? """
@@ -208,6 +205,14 @@ class Base(unittest.TestCase):
         actual_subti = fedmsg.meta.msg2subtitle(self.msg, **self.config)
         self._equals(actual_subti, self.expected_subti)
 
+    @skip_on(['msg', 'expected_subjective'])
+    def test_subjective(self):
+        """ Does fedmsg.meta produce the expected subjective message? """
+        for subject, expected in self.expected_subjective.items():
+            actual_subjective = fedmsg.meta.msg2subjective(
+                self.msg, subject=subject, **self.config)
+            self._equals(actual_subjective, expected)
+
     @skip_on(['msg', 'expected_link'])
     def test_link(self):
         """ Does fedmsg.meta produce the expected link? """
@@ -232,6 +237,12 @@ class Base(unittest.TestCase):
         actual_usernames = fedmsg.meta.msg2usernames(self.msg, **self.config)
         self._equals(actual_usernames, self.expected_usernames)
 
+    @skip_on(['msg', 'expected_agent'])
+    def test_agent(self):
+        """ Does fedmsg.meta produce the expected agent? """
+        actual_agent = fedmsg.meta.msg2agent(self.msg, **self.config)
+        self._equals(actual_agent, self.expected_agent)
+
     @skip_on(['msg', 'expected_packages'])
     def test_packages(self):
         """ Does fedmsg.meta produce the expected list of packages? """
@@ -258,20 +269,48 @@ class Base(unittest.TestCase):
 
 
 class TestUnhandled(Base):
-    expected_title = "unhandled_service.some_event"
+    expected_agent = None
+    expected_avatars = {}
+    expected_emails = {}
+    expected_icon = None
+    expected_link = ''
+    expected_long_form = ''
+    expected_objects = set()
+    expected_packages = set()
+    expected_secondary_icon = None
     expected_subti = ""
+    expected_title = "unhandled_service.some_event"
+    expected_usernames = set()
+
+    expected_subjective = {
+        'foo': expected_subti,
+        'bar': expected_subti,
+    }
+
     msg = {
         "topic": "org.fedoraproject.stg.unhandled_service.some_event"
     }
 
 
 class TestAnnouncement(Base):
-    expected_title = "announce.announcement"
-    expected_subti = 'hello, world.'
-    expected_long_form = 'hello, world.'
+    expected_agent = 'ralph'
+    expected_avatars = {}
+    expected_emails = {}
+    expected_icon = None
     expected_link = 'foo'
+    expected_long_form = 'hello, world.'
+    expected_objects = set()
+    expected_packages = set()
+    expected_secondary_icon = None
+    expected_subti = 'hello, world.'
+    expected_title = "announce.announcement"
     expected_usernames = set(['ralph'])
 
+    expected_subjective = {
+        'foo': expected_subti,
+        'bar': expected_subti,
+    }
+
     msg = {
         "i": 1,
         "timestamp": 1344352873.714926,
@@ -285,11 +324,24 @@ class TestAnnouncement(Base):
 
 
 class TestLoggerNormal(Base):
-    expected_title = "logger.log"
-    expected_subti = 'hello, world. (ralph)'
+    expected_agent = 'ralph'
+    expected_avatars = {}
+    expected_emails = {}
+    expected_icon = None
+    expected_link = ''
     expected_long_form = 'hello, world. (ralph)'
+    expected_objects = set()
+    expected_packages = set()
+    expected_secondary_icon = None
+    expected_subti = 'hello, world. (ralph)'
+    expected_title = "logger.log"
     expected_usernames = set(['ralph'])
 
+    expected_subjective = {
+        'ralph': 'you logged "hello, world."',
+        'lmacken': expected_subti,
+    }
+
     msg = {
         "i": 1,
         "timestamp": 1344352873.714926,
@@ -302,8 +354,18 @@ class TestLoggerNormal(Base):
 
 
 class TestLoggerJSON(Base):
-    expected_title = "logger.log"
+    expected_agent = 'root'
+    expected_avatars = {}
+    expected_emails = {}
+    expected_icon = None
+    expected_link = ''
+    expected_objects = set()
+    expected_packages = set()
+    expected_secondary_icon = None
     expected_subti = '<custom JSON message> (root)'
+    expected_title = "logger.log"
+    expected_usernames = set(['root'])
+
     expected_long_form = textwrap.dedent("""
     A custom JSON message was logged by root::
 
@@ -311,7 +373,11 @@ class TestLoggerJSON(Base):
             "foo": "bar"
         }
     """).strip()
-    expected_usernames = set(['root'])
+
+    expected_subjective = {
+        'foo': expected_subti,
+        'bar': expected_subti,
+    }
 
     msg = {
         "i": 1,
@@ -325,8 +391,8 @@ class TestLoggerJSON(Base):
 
 
 class ConglomerateBase(unittest.TestCase):
-    originals = None
-    expected = None
+    originals = Unspecified
+    expected = Unspecified
     maxDiff = None
 
     def setUp(self):
@@ -336,12 +402,12 @@ class ConglomerateBase(unittest.TestCase):
             invalidate_cache=True,
         )
         self.config['topic_prefix'] = 'org.fedoraproject'
-        self.config['topic_prefix_re'] = '^org\.fedoraproject\.(dev|stg|prod)'
+        self.config['topic_prefix_re'] = r'^org\.fedoraproject\.(dev|stg|prod)'
         fedmsg.meta.make_processors(**self.config)
 
         # Delete the msg_ids field because it is bulky and I don't want to
         # bother with testing it (copying and pasting it).
-        if self.expected:
+        if self.expected is not Unspecified:
             for item in self.expected:
                 if 'msg_ids' in item:
                     del item['msg_ids']
@@ -349,7 +415,9 @@ class ConglomerateBase(unittest.TestCase):
     @skip_on(['originals', 'expected'])
     def test_conglomerate(self):
         """ Does fedmsg.meta produce the expected conglomeration? """
-        actual = fedmsg.meta.conglomerate(self.originals, **self.config)
+        subject = getattr(self, 'subject', None)
+        actual = fedmsg.meta.conglomerate(
+            self.originals, subject, **self.config)
 
         # Delete the msg_ids field because it is bulky and I don't want to
         # bother with testing it (copying and pasting it).
@@ -360,10 +428,10 @@ class ConglomerateBase(unittest.TestCase):
 
         try:
             self.assertEquals(actual, self.expected)
-        except:
-            print "Failed at:"
-            print " ", self
-            print " ", os.path.relpath(inspect.getfile(type(self))[:-1])
+        except:  # noqa: E722
+            print("Failed at:")
+            print(" ", self)
+            print(" ", os.path.relpath(inspect.getfile(type(self))[:-1]))
             raise
 
 
@@ -375,29 +443,53 @@ class TestConglomeratorExtras(unittest.TestCase):
             invalidate_cache=True,
         )
         self.config['topic_prefix'] = 'org.fedoraproject'
-        self.config['topic_prefix_re'] = '^org\.fedoraproject\.(dev|stg|prod)'
+        self.config['topic_prefix_re'] = r'^org\.fedoraproject\.(dev|stg|prod)'
 
         self.conglomerator = fedmsg.meta.base.BaseConglomerator
 
     def test_list_to_series_simple(self):
-        original, expected = ['a', 'b', 'c'], "a, c, and b"
+        original, expected = ['a', 'b', 'c'], "a, b, and c"
         result = self.conglomerator.list_to_series(original)
-        eq_(result, expected)
+        self.assertEqual(result, expected)
 
     def test_list_to_series_single_duplicate(self):
         original, expected = ['a', 'a', 'a'], "a"
         result = self.conglomerator.list_to_series(original)
-        eq_(result, expected)
+        self.assertEqual(result, expected)
 
     def test_list_to_series_double_duplicate(self):
         original, expected = ['a', 'a', 'b', 'b'], "a and b"
         result = self.conglomerator.list_to_series(original)
-        eq_(result, expected)
+        self.assertEqual(result, expected)
 
     def test_list_to_series_backheavy_duplicate(self):
         original, expected = ['a', 'b', 'b', 'b'], "a and b"
         result = self.conglomerator.list_to_series(original)
-        eq_(result, expected)
+        self.assertEqual(result, expected)
+
+
+class TestMetaExtras(unittest.TestCase):
+    def test_graceful_keyerror(self):
+        """ Ensure that the graceful decorator handles KeyErrors """
+
+        expected = "default value"
+
+        @fedmsg.meta.graceful(lambda: expected)
+        def f(msg, **config):
+            raise KeyError("Who changed the API!?")
+
+        actual = f({})
+        self.assertEquals(actual, expected)
+
+    def test_graceful_other_error(self):
+        """ Ensure that the graceful decorator doesn't handle non-KeyErrors """
+
+        @fedmsg.meta.graceful(lambda: "default value")
+        def f(msg, **config):
+            raise Exception("This is worse than we thought.")
+
+        with self.assertRaises(Exception):
+            f({})
 
 
 if __name__ == '__main__':
